/*
 * Date: 2020-05-23
 * Project: 18F45K20_UART
 * 
 * Description:
 * 
 *  MPLAB Code Configurator (MCC) project that implements 
 *  an Interrupt Service Routine (ISR) for UART receive 
 *  interrupts and a polled transmit data output function.
 * 
 *  This may only work in real hardware because the MPLABX
 *  simulation tool is really kind of lame.
 * 
 */
#define VERSION_STRING "18F45K20 UART Test v1.0\r\n"

/**
  Generated Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using PIC10 / PIC12 / PIC16 / PIC18 MCUs

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  PIC10 / PIC12 / PIC16 / PIC18 MCUs - 1.81.0
        Device            :  PIC18F45K20
        Driver Version    :  2.00
*/

/*
    (c) 2018 Microchip Technology Inc. and its subsidiaries. 
    
    Subject to your compliance with these terms, you may use Microchip software and any 
    derivatives exclusively with Microchip products. It is your responsibility to comply with third party 
    license terms applicable to your use of third party software (including open source software) that 
    may accompany Microchip software.
    
    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY 
    IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS 
    FOR A PARTICULAR PURPOSE.
    
    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP 
    HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO 
    THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL 
    CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT 
    OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS 
    SOFTWARE.
*/

#include "mcc_generated_files/mcc.h"

/*
 * Send buffer out the UART
 */
void DumpBuffer(void *Buffer, uint8_t Length)
{
    uint8_t *Buf = (uint8_t *)Buffer;
    
    if(Buf && Length) do
    {
        EUSART_Write(*Buf++);
    } while(--Length);
    while(!EUSART_is_tx_done());
}

/*
                         Main application
 */
void main(void)
{
    uint8_t UART_InputBuffer[80];
    uint8_t Index;
    uint8_t Count;
    
    // Initialize the device
    SYSTEM_Initialize();

    // If using interrupts in PIC18 High/Low Priority Mode you need to enable the Global High and Low Interrupts
    // If using interrupts in PIC Mid-Range Compatibility Mode you need to enable the Global and Peripheral Interrupts
    // Use the following macros to:

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();

    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();

    DumpBuffer(VERSION_STRING, sizeof(VERSION_STRING)-1);
    while (1)
    {
        // Add your application code
        Index = 0;
        Count = 0;
        while(EUSART_is_rx_ready())
        {
            UART_InputBuffer[Index++] = EUSART_Read();
            Count++;
            if(Index >= sizeof(UART_InputBuffer))
            {
                break;
            }
        }
        if(Count)
        {
            if(EUSART_is_tx_ready())
            {
                DumpBuffer(UART_InputBuffer,Count);
            }
        }
    }
    /*
     * These function calls suppress annoying compiler warnings
     * that are a result of MCC generated code that is not used.
     */
    EUSART_get_last_status();
    PIN_MANAGER_IOC();
}
/**
 End of File
*/